package orderprocessing;

import java.io.Serializable;
import java.util.StringTokenizer;

/** This class maintains information for an address

	History	
	date		description
	-----		-----------
	11/12/99	Creation, break out from customer class
				
 This class represents addresses, usually customer addresses
*/
public class Address implements Serializable, Versionable
{	/** Create an address from a set of strings representing the parts of 
	* the address
	* @param name the customer name
	* @param street the street address of the customer
	* @param city the city of the customer
	* @param state the state (US) of the customer
	* @param zip the zip code of the customer
	*/
    public Address(String name, String street, String city, String state, String zip)
	{	if (name != null) this.name = name;
	    if (street != null) this.street = street;
	    if (city != null) this.city   = city;
	    if (state != null) this.state  = state;
	    if (zip != null) this.zip    = zip;
	}
	
	/** Create an address from a single encoded string
	* @param all the complete address of the customer beginning with the name
	* @parameter separator the separator character between fields of all
	*/
	public Address(String all, String separator)
	{	StringTokenizer tok = new StringTokenizer(all, separator);
		name = tok.nextToken();
		street = tok.nextToken();
		city = tok.nextToken();
		state = tok.nextToken();
		zip = tok.nextToken();
	}
	
	public Object newVersion(Object information){return null; }
	
	public void display()
	{	System.out.println(this); // Automatically uses toString.
	}
	
	// Accessors
	public String   name()       { return name; }
	public String   street()     { return street; }
	public String   city()       { return city;   }
	public String   state()      { return state;  }
	public String   zip()        { return zip;    }
	
	// Mutators
	public void name(String name) 	 { if (name   != null) this.name = name; }
	public void street(String street) { if (street != null) this.street = street; }
	public void city(String city)     { if (city   != null) this.city   = city;   }
	public void state(String state)   { if (state  != null) this.state  = state;  }
	public void zip(String zip)       { if (zip    != null) this.zip    = zip;    }
	
	public String toString()
	{	return  name + " " + street + " " + city + ", " +
	            state + " " + zip;
	}
	 
	private String        name       = "";   
	private String        street     = "";
	private String        city       = "";
	private String        state      = "";
	private String        zip        = "";
}

