package orderprocessing;

/*
	History	
	date		description
	-----		-----------
	11/12/99	Creation of 2 billing terms. 
*/

/** This class represents various billing terms in use by the firm
* it exports constant terms objects for all allowable terms. 
*/
public class BillingTerms implements java.io.Serializable, Versionable
{	public static final int NET = 0;
	public static final int EOM = 1;
	private static final int minDays = 5;
	private static final int maxDays = 120;
	
	public static final BillingTerms NET_10_DAYS   =  new BillingTerms(NET, 10);
   public static final BillingTerms NET_30_DAYS   =  new BillingTerms(NET, 30);
    
    public BillingTerms(int terms, int days) 
    {		if(days<minDays) days = minDays;
    		if(days>maxDays) days = maxDays;
    		this.terms = terms; 
    		this.days = days;
    }

    public int billingTerms() {return terms;}
    
    public int days(){return days;}
    
	public Object newVersion(Object information){return null; }
	
    /** return the due date for payment on the associated order given the invoice
    * date.
    * @param invoiceDate the date that the order was invoiced.
    * @return the due date for payment according to these terms.
    */
    public CalculationCalendar dueDate(CalculationCalendar invoiceDate)
    {	CalculationCalendar result = (CalculationCalendar) invoiceDate.clone();
    	if(terms == NET)
    		result.add(CalculationCalendar.DATE, days);
    	else if(terms == EOM)
    	{	result.roll(CalculationCalendar.MONTH, true);
    		result.set(CalculationCalendar.DATE, days);
    	}
    	return result;
    }
    
    public String toString(){return "Billing terms " + (terms == NET?"Net":"EOM") + days;}
    
    public boolean equals(Object other)
    {	if(!(other instanceof BillingTerms))return false;
    	BillingTerms o = (BillingTerms)other;
    	return terms == o.terms && days == o.days;
    }
    
    public int hashCode() {return terms*997 + days*13;}
    
    public void display(){System.out.println(this);}
    
    private int terms = NET;
    private int days = 0;
    
}

