package orderprocessing;
import java.util.StringTokenizer;

import java.io.*;

/** This class drives the customer transactions from an input file.
    Each transaction is passed to customer processor.

    History
    Date      Description
    --------  -----------
    12/12/99  Read customer transaction input file and parse its contents.
    12/14/99  Parse the transaction input with "=blank" if field is empty.
*/

public class CustomerMaintenance
{
  /** This is the main action: parsing transaction input file and pass the
  *   info for customerProcessor's newTransaction method.
  */
	public static void fireCustomerTrans() 
    {	String transaction = "";
    	String customer = "";
    	String extraInfo = "";

		try
		{	while ((transaction = in.readLine()) != null)
			{	System.out.println("c---> " + transaction); // Debug code.
        // Only read more lines of info. if trans is new or edit.
        if ( (transaction.indexOf("new") >= 0) ||
             (transaction.indexOf("edit") >= 0) ) {
				  customer = in.readLine();
				  String street = in.readLine();
          String city = in.readLine();
          String state = in.readLine();
          String zip = in.readLine();
          customer = checkBlanks( customer + "~" + street +  "~" + city + "~" +
                                  state + "~" + zip, "~" );

          String shipTo = in.readLine();
			    street = in.readLine();
          city = in.readLine();
          state = in.readLine();
          zip = in.readLine();
				  String billTerms = in.readLine();
          if ( shipTo.indexOf("=blank") == 0 ) {
            if ( billTerms.indexOf("=blank") == 0 )
              extraInfo = "";
            else
              extraInfo = billTerms;
          } else {
            if ( billTerms.indexOf("=blank") == 0 )
              billTerms = "";
            extraInfo = shipTo + "~" + street + "~" + city +
                        "~" + state + "~" + zip + "~" + billTerms;
          }
          System.out.println("c---> " + customer); // Debug code.
          System.out.println("c---> " + extraInfo); // Debug code.
          System.in.read(); // Debug code -- prevent console disappear.
          customerProcessor.newTransaction(transaction, customer, extraInfo);
        } else
          customerProcessor.newTransaction(transaction);
			}
			in.close();
		} catch(IOException e){
      System.out.println("Failed reading customer transactions.");
    }
	}

  /** Checks for "=blank" in the billing address.  If "=blank" is found,
  *   the whole field is removed.  The resulted string is returned.
  * @param withBlanks  The input string which might contain "=blank"s
  * @param delimeter  Seperator between the fields.
  */
  private static String checkBlanks( String withBlanks, String delimiter ) {
    StringTokenizer tok = new StringTokenizer( withBlanks, delimiter );
    int count = tok.countTokens();
    String anyBlanks = tok.nextToken();
    String realNulls = anyBlanks.indexOf("=blank") == 0 ? "" : anyBlanks;

    for (int i=1; i<count; i++) {
      anyBlanks = tok.nextToken();
      if (anyBlanks.indexOf("=blank") == 0)
        anyBlanks = "";
      realNulls += delimiter + anyBlanks;
    }

    return realNulls;
  }

	private static BufferedReader in;

	public static void main(String args[])
	{	try
		{	in = new BufferedReader(new FileReader(args[0]));
		}
		catch (IOException e)
		{	System.out.println("Failed opening customer transaction file.");
			System.exit(1);
		}
    	CustomerProcessor.open();
	  	fireCustomerTrans();
    	CustomerProcessor.close();
		System.out.println( "All done!" );
	}

	private static CustomerProcessor customerProcessor =  CustomerProcessor.customerProcessor;
}

/* custrans.dat sample contents
new
Smith and Co.
123 Main Street
Peoria
Il
11212
Smith and Co. Warehouse
777 Big St
Big City
IL
33333
NET 10
new
Smith and Son
9 Very Long Ave.
Virtual City
NY
12601
Smith and Son Store
456 Ship St.
Warehouse City
NY
12443
NET 5

The format is:
Transaction on one line with or without an ID
Customer name
Customer billing street address
Customer billing city
Customer billing state
Customer billing zip
Customer shipping street address
Customer shipping city
Customer shipping state
Customer billing zip
Customer billing term = NET or EOM with Days range 5-120

Note: If a line is blank (no value for the field), it is presented by string
"=blank" at BEGINNING of the line.  After that, any comments can be added
and will be ignored.
*/

/*
 Compile command is:
javac  -d . CustomerGenerator.java CustomerProcessor.java Customer.java dBMaintenanceIF.java

 Run command is:
java orderprocessing.CustomerGenerator custrans.dat
*/


