package orderprocessing;

import java.io.*;
//import java.text.*;
import java.util.*;


/** This class manages inventory and responds to requests from an OrderProcessor
*/

public class InventoryManager
{	public InventoryManager(){}

	/** Tells if a given order can be filled now
	*	@ param order The OrderItem to be checked.
	*	@ return an OrderItem with the quantity that can be shipped. 
	*/
	public OrderItem available(OrderItem item)
	{	InventoryItem product = (InventoryItem)allInventory.get(item.itemNumber());
		if (product == null) return null;
		if(product.quantity() >= item.quantity())
		{	product.decrement(item.quantity());
			return (OrderItem)item.clone();
		}
		else
		{	int shipable = product.decrement(item.quantity());
			// MORE -- enter new order for excess items
			return new OrderItem(shipable, item.product(), item.date());
		}
	}
	
	/** Following methods are used to manipulate the inventory in memory
   */	
   public static InventoryItem newInventoryItem(Product p, int quantity)
   {	InventoryItem i = new InventoryItem(p, quantity);
   	return i;
   }
   
   public InventoryItem get(Product p) // may be null
   {	return (InventoryItem)allInventory.get(p.itemNumber());
   }
   
   public InventoryItem get(String itemNumber) // may be null
   { return (InventoryItem)allInventory.get(itemNumber);
   }
   
	public static Enumeration inventory()
    {		return allInventory.elements();	
    }

  public static final void display(String ItemID)
    {		InventoryItem item = (InventoryItem)allInventory.get(ItemID);
			if (item != null) item.display();
    }
    
  public static final void displayAll()
    {	System.out.println("Inventory List");
    	Enumeration e = inventory();
        while (e.hasMoreElements()) 
        {   InventoryItem item = (InventoryItem)e.nextElement();
            item.display();
        }
    }
    
  public static final void close(ObjectOutputStream out)
	    throws IOException
	{	out.writeObject(allInventory);	
	}

  public static final void open(ObjectInputStream in) 
      throws IOException, ClassNotFoundException
	{	allInventory = (Hashtable)in.readObject();
	}  


	public static InventoryManager standardInventoryManager = new InventoryManager();
	private static Hashtable allInventory = new Hashtable();

	
	//--------------------------------------------------------------------------

	public static class InventoryItem implements Serializable, Versionable
	{
		public InventoryItem(Product p){this(p, 0);}
		
	  /*
	  * Create an inventory item for a given quantity of a certain product item
	  * @param quantity The quantity ordered.
	  * @param p The item desired.
	  */
		public InventoryItem(Product p, int quantity)
		{	this.quantity = quantity;
			this.product = p;
   			allInventory.put(p.itemNumber(), this);
		}
	
		public void increment(int amount){quantity += amount;}
	
		public int decrement(int amount)
		{	if(amount > quantity)
			{	int result = quantity;
				quantity = 0;
				return result;
			}
			quantity -= amount;
			return amount;
		}
	
		public int quantity(){return quantity;}
		public String itemNumber(){return product.itemNumber();}
		public Product product(){return product;}

		public boolean equals(Object o)
		{	if (!(o instanceof InventoryItem))	return false;
			InventoryItem oi = (InventoryItem)o;
			return  product.itemNumber() == oi.product.itemNumber();
		}
	
		public int hashCode(){return product.itemNumber().hashCode();}
	
		public Object newVersion(Object information){return null; }
	
		public void display()
		{	System.out.println("\t" +product.itemNumber()+ "\t" + quantity + "\t\t" );
		}
	
		public String toString() 
		{	return quantity + " " + product.itemNumber();
		}
		
		private int quantity;
		private Product product;

	}
	
}
