package orderprocessing;
/*
	History	
	date		description
	-----		-----------
	11/12/99	Change date to GreforianCalendar,
	            format output
*/

import java.util.Vector;
import java.util.Enumeration;
import java.io.Serializable;
import java.text.*;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.util.Hashtable;

/** This class represents what will ship from a customer order. 
*/

public class Invoice implements Serializable, Versionable
{	/** Create an Invoice for a particular customer and a sequence of items ordered
	* @param who The customer ordering these items
	* @param what A vector of OrderItems representing the items ordered
	*/
	public Invoice(Order order, Vector shipableItems)
	{	this.shipableItems = (Vector) shipableItems.clone();
		shipDate = order.requestedShippingDate(); 
		this.order = order;
		allInvoices.put(key, this);
	}
	
	public Object newVersion(Object information){return null; }
	
	public Customer customer(){return order.customer();}
	public Enumeration shipableItems(){return shipableItems.elements();}
	public CalculationCalendar shipDate(){return shipDate;}
	
	/** Return the invoice number
	*/
	public String key(){return key;}
	
	public CalculationCalendar paymentDueDate()
	{	return order.billingTerms().dueDate(shipDate);
	}
	
	/** Display the information about shipableItems will ship, including extended prices and a total
	*/
	public void display()
	{	System.out.println("\t\t\t------- Invoice -------");
	    System.out.println("Invoice No: " + key);
	    //System.out.println("Customer : " );
	    order.customer().display();
		Enumeration items = shipableItems();
		double total = 0.0;
	    
	    System.out.println("\n\n\tQty\tItem No. " +
	            "\tDescription\tsub-total\n");
	           
	    while(items.hasMoreElements())
		{	OrderItem ordered = (OrderItem)items.nextElement();
			ordered.displayExtended();
			total += ordered.quantity() * ordered.unitPrice();
		}

		System.out.println("\n\t\t\t\tInvoice Total: " 
		    + NumberFormat.getInstance().format(total));
	    System.out.println("-----------------------------------------------------------------");
	}
	
	public boolean equals(Object other)
	{	if(!(other instanceof Invoice))return false;
		return key.equals(((Invoice)other).key);
	}
	
	public int hashCode()
	{	return key.hashCode()*997;
	}
	
	public String toString(){return "Invoice with key: " + key;}

	private Vector shipableItems; // Vector of OrderItems
	private String key = IDGenerator.nextID();
	private CalculationCalendar shipDate;
	private Order order;

	public static class IDGenerator 
	{	public static String nextID()
	    {	return ID_PREFIX + Long.toString(nextID++);
	    }
	    
	    private static long             nextID = 6001;
	    private final static String     ID_PREFIX = "I";
	}
	
	private static Hashtable allInvoices = new Hashtable();
	public static Enumeration invoices()
	{	return allInvoices.elements();
	}
	public static final Object remove(String invoiceID)
	{	return allInvoices.remove(invoiceID);
	}
	public static final void close(ObjectOutputStream out)throws IOException
	{	out.writeLong(IDGenerator.nextID);
		out.writeObject(allInvoices);
	}
	public static final void open(ObjectInputStream in) 
	throws IOException, ClassNotFoundException
	{	IDGenerator.nextID = in.readLong();
		allInvoices = (Hashtable)in.readObject();
	}
		
}
