package orderprocessing;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.GregorianCalendar;

/** This class fires orders from customers into the OrderProcessor. It simply opens a 
* file and generates an order from the contents.
*/

public class OrderGenerator 
{	public OrderGenerator(String filename)
	{	try
		{	in = new BufferedReader(new FileReader(filename));
		}
		catch (IOException e)
		{	System.out.println("Failed opening order file.");
			System.exit(1);
		}
	}
	
	public void fireOrders()
	{	String customer, shipping, terms;
		Vector items = new Vector();
		String str;
		CalculationCalendar orderDate, requestedShipDate, cancelDate;
		try
		{	while ((customer = in.readLine()) != null)
			{	//System.out.println("c---> " + customer); // Debug code. 
			  	shipping = in.readLine();
				terms = in.readLine();
				orderDate = new CalculationCalendar();//now
				try
				{	orderDate.setTime((new SimpleDateFormat("MMMM d, yyyy")).parse(in.readLine()));
				}
				catch (ParseException pe)
				{	System.out.println("Illegal date value");
				}
				requestedShipDate = new CalculationCalendar(); // now
				try
				{	requestedShipDate.setTime((new SimpleDateFormat("MMMM d, yyyy")).parse(in.readLine()));
				}
				catch (ParseException pe)
				{	System.out.println("Illegal date value");
				}
				cancelDate = new CalculationCalendar(); // now
				try
				{	cancelDate.setTime((new SimpleDateFormat("MMMM d, yyyy")).parse(in.readLine()));
				}
				catch (ParseException pe)
				{	System.out.println("Illegal date value");
				}
				while(((str = in.readLine()) != null) && !str.trim().equals( "" ) )
				{	//System.out.println("i---> " + str); // Debug code. 
					items.addElement(str);
				}
				Order order = orderProcessor.newOrder(customer, terms, shipping, items, orderDate, requestedShipDate, cancelDate);
				if(order != null)
					order.display();
				else
					System.out.println("Illegal order for customer: " + customer);
				items.setSize(0);
			}
			in.close();
			
		} catch(IOException e){System.out.println("Failed reading orders.");}
	}
	
	private BufferedReader in;

	public static void main(String args[]) 
	{	OrderProcessor.open();
	    new OrderGenerator(args[0]).fireOrders();
	    OrderProcessor.close();
		System.out.println( "All done!" );
		try 
		{	System.in.read(); // prevent console window from going away
		} catch (java.io.IOException o) {}
	}
	
	private OrderProcessor orderProcessor =  OrderProcessor.orderProcessor;
}

/* orders.dat sample contents

C5001
defaultaddress
defaultterms
August 17, 1999
August 27, 1999
September 15, 1999
P2001 15 
P2001 22 

C5003
Marys Mfg.~8726 Oak~Chelmsford~MA~99999
30
August 19, 1999
August 25, 1999
August 30, 1999
P2002 5 
P2004 22 

C5003
Marys Mfg.~8726 Oak~Chelmsford~MA~99999
30
August 19, 1999
August 25, 1999
August 30, 1999
P2002 5 
P2002 5 
P2003 5 

C5002
Marys Mfg.~8726 Oak~Chelmsford~MA~99999
30
August 19, 1999
August 25, 1999
August 30, 1999
P2001 5 
P2002 22

The format is:
Customer number on first line (must be valid)
Order ship address or a word containing default
Billing terms for this order or a word conaining default
An order date.
A requested ship date.
A requested canceldate
Each item ordered on one line with item number, quantity separated by spaces
blank line between orders
*/

/*
 Compile command is:
javac  -d . OrderGenerator.java OrderProcessor.java Order.java InventoryManager.java Customer.java OrderItem.java Invoice.java

 Run command is:
java orderprocessing.OrderProcessor orders.dat
*/

