package orderprocessing;

/*
	History	
	date		description
	-----		-----------
	11/12/99	Add item description
*/

import java.io.Serializable;
import java.text.*;

/** This class contains information about a single item on a customer order
* Note that this class implements the Immutable Object design pattern. 
*/

public class OrderItem implements Serializable, Cloneable, Versionable
{	/** Create an order item for a given quantity of a certain item
	* Before creating an order item, you must verify that the itemNumber is valid
	* for a product we ship. 
	* @param quantity The quantity ordered.
	* @param itemNumber The item number of the Product desired.
	* @param unitPrice The price per unit for this item. 
	* @param description - Item description 
	*/
	public OrderItem(int quantity, Product item, CalculationCalendar date)
	{	this.quantity = quantity;
		this.item = item;
		this.date = date;
	}
	
	public Object newVersion(Object information){return null; }
	
	public int quantity(){return quantity;}
	public Product product(){return item;}
	public String itemNumber(){return item.itemNumber();}
	public CalculationCalendar date(){ return date; }
	public double unitPrice()
	{	return item.unitPrice(date);
	}
	public double extendedPrice(){return unitPrice() * quantity;}
	public String description()
	{	return item.description();
	}
	
	public boolean equals(Object o)
	{	if (!(o instanceof OrderItem)) return false;
		OrderItem oi = (OrderItem)o;
		return quantity == oi.quantity && itemNumber() == oi.itemNumber();
	}
	
	public int hashCode(){return (int)((quantity +1) * itemNumber().hashCode());}
	// Guarantee that two "equals" OrderItems have the same hash code. 
	
	public void display()
	{	
	    //System.out.println(this);
	    System.out.println("\t" +quantity+ "\t" + 
	                       itemNumber() + "\t\t" + 
	                       description() + "\t\t" +
	                       NumberFormat.getInstance().format(unitPrice()));
	    
	}
	
	public void displayExtended()
	{	
	    System.out.println("\t" +quantity+ "\t" + 
	                       itemNumber() + "\t\t" + 
	                       description() + "\t\t" +
	                       NumberFormat.getInstance().format(unitPrice() * quantity));
	}
	
	public Object clone(){return new OrderItem(quantity, item, date);}
	
	public String toString(){ return quantity + " " + itemNumber() + " " + unitPrice();}
	
	private int quantity;
	private Product item;
	private CalculationCalendar date;
}


