package orderprocessing;

import java.text.*;
import java.util.*;
import java.io.*;

public class Product implements Serializable 
 {  public Product(String description, CalculationCalendar date, double price) 
	{	this.itemNumber = IDGenerator.nextID();
		this.description = description;
		addPrice(date, price);
		allProducts.put(itemNumber, this);
	}
	
	public String itemNumber() { return itemNumber; }
	
	public String description() { return description; }
	
	public void modify(String description) 
	{    this.description = description;
	}
   
	public String toString()
    {    String temp = itemNumber + "\t" + description;
        Enumeration e = priceSchedule.elements();
        while(e.hasMoreElements())
            temp = temp + "\n\t" + e.nextElement().toString();
        return temp;
    }
    
    public boolean equals(SortableObject o) 
    {   if (!(o instanceof Product)) return false;
        Product oi = (Product)o;
        return itemNumber == oi.itemNumber;
    }
    
    public int hashCode(){return itemNumber.hashCode();}
    
    public void addPrice(CalculationCalendar date, double price) 
    {   Price o = new Price(date, price);
        priceSchedule.add(o);
    }

    public boolean deletePrice(CalculationCalendar date) 
    {   Price o = new Price(date);
        return priceSchedule.remove(o);
    }

    public double unitPrice(CalculationCalendar saleDate) 
    {   Price o = new Price(saleDate);
        if ( priceSchedule.size() == 0 )
        { 	System.out.println("empty pricelist");
        	return -1;
        }
        int i = priceSchedule.binarySearch(o);
        if ( priceSchedule.size() == i ) --i;
        Price p = (Price)priceSchedule.elementAt(i);
        if (p.equals(o)) 
        {   return p.price();
        } else 
        {   if ( i < 0 )
        	{	System.out.println("No valid price.");
        		return -1;
        	}
            p = (Price)priceSchedule.elementAt(i);
            return p.price();
        }
    }

    public boolean modifyPrice(CalculationCalendar date, double price) 
    {   Price o = new Price(date);
        Price d = (Price)priceSchedule.search(o);
        if ( d instanceof Price) 
        {  d.modify(price);
           return true;
        } else
           return false;
    }
    
	private String   itemNumber;
	private String description;
    private SortedList priceSchedule = new SortedList();
    private static Hashtable allProducts = new Hashtable();
   
	public static class IDGenerator 
	{	public static String nextID()
	    {	return ID_PREFIX + Long.toString(nextID++);
	    }
	    
	    private static long             nextID = 2001;
	    private final static String     ID_PREFIX = "P";
	}
	
	public static Product get(String idNumber)
	{	return (Product)allProducts.get(idNumber);
	}
	
	public static Enumeration elements(){return allProducts.elements();}
	
	public static Product remove(String itemNumber)
	{	return (Product)allProducts.remove(itemNumber);
	}
	
	public static final void close(ObjectOutputStream out)throws IOException
	{	out.writeLong(IDGenerator.nextID);
		out.writeObject(allProducts);
	}
	public static final void open(ObjectInputStream in) 
	throws IOException, ClassNotFoundException
	{	IDGenerator.nextID = in.readLong();
		allProducts = (Hashtable)in.readObject();
	}
	

}


